package java.net;

import sun.net.ResourceManager;
import java.io.IOException;
import java.io.FileDescriptor;

class TwoStacksPlainSocketImpl extends AbstractPlainSocketImpl
{
    private FileDescriptor fd1;
    private InetAddress anyLocalBoundAddr;
    private int lastfd;
    private final boolean exclusiveBind;
    private boolean isReuseAddress;
    
    public TwoStacksPlainSocketImpl(boolean exclusiveBind, boolean b) {
        super(b);
        this.anyLocalBoundAddr = null;
        this.lastfd = -1;
        this.exclusiveBind = exclusiveBind;
    }
    
    @Override
    protected synchronized void create(boolean b) throws IOException {
        this.fd1 = new FileDescriptor();
        try {
            super.create(b);
        }
        catch (IOException ex) {
            this.fd1 = null;
            throw ex;
        }
    }
    
    @Override
    protected synchronized void bind(InetAddress anyLocalBoundAddr, int n) throws IOException {
        super.bind(anyLocalBoundAddr, n);
        if (anyLocalBoundAddr.isAnyLocalAddress()) {
            this.anyLocalBoundAddr = anyLocalBoundAddr;
        }
    }
    
    @Override
    public Object getOption(int n) throws SocketException {
        if (this.isClosedOrPending()) {
            throw new SocketException("Socket Closed");
        }
        if (n == 15) {
            if (this.fd != null && this.fd1 != null) {
                return this.anyLocalBoundAddr;
            }
            InetAddressContainer inetAddressContainer = new InetAddressContainer();
            this.socketGetOption(n, inetAddressContainer);
            return inetAddressContainer.addr;
        }
        else {
            if (n == 4 && this.exclusiveBind) {
                return Boolean.valueOf(this.isReuseAddress);
            }
            return super.getOption(n);
        }
    }
    
    @Override
    void socketBind(InetAddress inetAddress, int n) throws IOException {
        this.socketBind(inetAddress, n, this.exclusiveBind);
    }
    
    @Override
    void socketSetOption(int n, boolean isReuseAddress, Object o) throws SocketException {
        if (n == 4 && this.exclusiveBind) {
            this.isReuseAddress = isReuseAddress;
        }
        else {
            this.socketNativeSetOption(n, isReuseAddress, o);
        }
    }
    
    @Override
    protected void close() throws IOException {
        synchronized (this.fdLock) {
            if (this.fd != null || this.fd1 != null) {
                if (!this.stream) {
                    ResourceManager.afterUdpClose();
                }
                if (this.fdUseCount == 0) {
                    if (this.closePending) {
                        return;
                    }
                    this.closePending = true;
                    this.socketClose();
                    this.fd = null;
                    this.fd1 = null;
                    return;
                }
                else if (!this.closePending) {
                    this.closePending = true;
                    --this.fdUseCount;
                    this.socketClose();
                }
            }
        }
    }
    
    @Override
    void reset() throws IOException {
        if (this.fd != null || this.fd1 != null) {
            this.socketClose();
        }
        this.fd = null;
        this.fd1 = null;
        super.reset();
    }
    
    @Override
    public boolean isClosedOrPending() {
        synchronized (this.fdLock) {
            if (this.closePending || (this.fd == null && this.fd1 == null)) {
                return true;
            }
            return false;
        }
    }
    
    static native void initProto();
    
    @Override
    native void socketCreate(boolean p0) throws IOException;
    
    @Override
    native void socketConnect(InetAddress p0, int p1, int p2) throws IOException;
    
    native void socketBind(InetAddress p0, int p1, boolean p2) throws IOException;
    
    @Override
    native void socketListen(int p0) throws IOException;
    
    @Override
    native void socketAccept(SocketImpl p0) throws IOException;
    
    @Override
    native int socketAvailable() throws IOException;
    
    @Override
    native void socketClose0(boolean p0) throws IOException;
    
    @Override
    native void socketShutdown(int p0) throws IOException;
    
    native void socketNativeSetOption(int p0, boolean p1, Object p2) throws SocketException;
    
    @Override
    native int socketGetOption(int p0, Object p1) throws SocketException;
    
    @Override
    native void socketSendUrgentData(int p0) throws IOException;
    
    static {
        initProto();
    }
}
